<?php


/**
 * Class WP_Persistent_Login_Active_Logins
 *
 * @since 2.0.0
 */
class WP_Persistent_Login_Active_Logins_Premium extends WP_Persistent_Login_Active_Logins {

    protected $limit;


	/**
	 * __construct
	 *
     * @since  2.0.0
	 * @return void
	 */
	public function __construct() {

        $settings = new WP_Persistent_Login_Settings_Premium();
        $this->limit = (int) $settings->get_active_logins_allowed();

		parent::__construct();

        // create custom blocks
        add_action( 'init', array($this, 'init_blocks') );

        // create shotcode fallback for custom blocks
        add_shortcode( 'persistent-login-manage-sessions', array($this, 'manage_sessions_shortcode') );
        add_shortcode( 'wppl-manage-sessions', array($this, 'manage_sessions_shortcode') );
        add_shortcode( 'wppl-maximum-logins-control', array($this, 'maximum_logins_control_shortcode') );

        // handle ending sessions when users reach the maximum login limit
        add_action( 'admin_post_wppl_end_session', array($this, 'manage_end_session'));
        add_action( 'admin_post_nopriv_wppl_end_session', array($this, 'manage_end_session'));

	}

    
    /**
     * is_user_subject_to_active_login
     *
     * @param  int $user_id
     * @return bool
     */
    public function is_user_subject_to_active_login($user_id) {

        $settings = new WP_Persistent_Login_Settings_Premium();
        $active_login_roles = $settings->get_active_login_roles();

        // fetch the WP_User object
        $user = get_user_by( 'id', (int) $user_id );
                                    
        // check if the users role is checked for active login
        if( is_array($user->roles) ) {
            $role_check = array_intersect($user->roles, $active_login_roles);
            if( !empty($role_check) ) {
                $role_check = true;
            } else {
                $role_check = false;
            }
        } else {
            $role_check = in_array($user->roles, $active_login_roles);
        }

        return $role_check;
        
    }

    
    /**
     * redirect_to_login_management
     *
     * @param  int $user_id
     * @return void
     */
    private function redirect_to_login_management($user_id) {

        $settings = new WP_Persistent_Login_Settings_Premium();
        
        $redirect_id = $settings->get_maximum_logins_redirect();
        $redirect_url = get_the_permalink($redirect_id);
        $nonce = wp_create_nonce( 'wppl_end_session_'.$user_id );
        $redirect_query = "?uid=$user_id&wppl_nonce=$nonce";
        $redirect = $redirect_url.$redirect_query;

        wp_safe_redirect($redirect, 302, 'WordPress Persistent Login');
        exit;

    }


    /**
	 * Validate if the maximum active logins limit reached.
	 *
	 * This check happens only after authentication happens
	 *
	 * @param boolean $check    Whether the passwords match..
	 * @param string  $password Plaintext user's password.
	 * @param string  $hash     Hash of the user's password to check against.
	 * @param int     $user_id  User ID.
	 *
	 * @since  2.0.0
	 * @access public
	 *
	 * @return bool
	 */
	public function validate_block_logic( $check, $password, $hash, $user_id ) {

		// If the validation failed already, bail.
		if ( ! $check ) {
			return false;
		}

        $subject_to_active_logins = $this->is_user_subject_to_active_login($user_id);

        if( $subject_to_active_logins == true ) {

            $settings = new WP_Persistent_Login_Settings_Premium();
            $limit_active_logins = $settings->get_limit_active_logins();
            
            // if we should limit the number of active logins, 
            if( $limit_active_logins === '1' ) {

                // if the limit is exceeded, start ending sessions
                if ( $this->reached_limit( $user_id ) ) {        
                    
                    $original_redirect = $_REQUEST['redirect_to'];
                    setcookie('wppl-redirect', $original_redirect, 0, '/');
                    set_transient( 'wppl-redirect-'.$user_id, $original_redirect, HOUR_IN_SECONDS );

                    // fetch the user object
                    $user = get_user_by('ID', $user_id);

                    // get the total number of invalid sessions
                    $invalid_sessions = $this->get_invalid_session_count($user_id);

                    // check the limit reached logic
                    $limit_reached_logic = $settings->get_limit_reached_logic();

                    switch($limit_reached_logic) {

                        case 'automatic' :

                            // get all users sessions, oldest first
                            $sessions = $this->get_user_sessions($user, SORT_ASC);

                            // setup a session manager
                            $session_manager = new WP_Persistent_Login_Manage_Sessions($user_id);

                            // if there are invalid sessions remove them
                            if( $invalid_sessions > 0 ) {

                                $session_tokens = array_keys($sessions);
                                
                                // remove all invalid sessions, leaving only the limit
                                for( $i = 0; $i < $invalid_sessions; $i++ ) {
                                    $session_token = $session_tokens[$i];
                                    $session_manager->persistent_login_update_session($session_token);
                                }
                            
                            }
                            break;

                        case 'manual' :

                            $this->redirect_to_login_management($user_id);

                    }     
                    
                }
            }
        
        }

		return true;

	}

    
    /**
     * get_invalid_session_count
     *
     * @param  int $user_id
     * @return int
     */
    protected function get_invalid_session_count($user_id) {

        $user = get_user_by('ID', $user_id);

        // get all users sessions, oldest first
        $sessions = $this->get_user_sessions($user, SORT_ASC);

        // count the current sessions
        $session_count = $this->get_login_count($user_id);

        // invalid sessions = current sessions, plus this login, minus the limit
        $invalid_sessions = ($session_count+1) - $this->limit;

        return $invalid_sessions;

    }


    private function get_sessions_to_end_message($user_id) {

        $sessions_to_end = $this->get_invalid_session_count($user_id);

        if( $sessions_to_end > 1 ) {
            $devices = __('devices', WPPL_TEXT_DOMAIN);
        } else {
            $devices = __('device', WPPL_TEXT_DOMAIN);
        }

        $sessions_to_end_message = sprintf(
            __(
                '<p>You have reached your maximum number of active logins.</p>
                <p>Please logout of %d %s from the list below to continue.</p>', 
                WPPL_TEXT_DOMAIN
            ), 
            $sessions_to_end,
            $devices
        );

        return $sessions_to_end_message;

    }


    public function manage_end_session() {

        $nonce_check = wp_verify_nonce( $_POST['_wpnonce'], 'wppl_end_session' );
        if( $nonce_check === false ) {
            echo 'Error 879: Sorry, there was an error, please try again.';
            die;
        }

        if( !isset($_POST['token']) || !isset($_POST['uid']) ) {
            echo 'Error 685: Sorry, there was an error, please try again.';
            die;
        }

        $token = sanitize_text_field($_POST['token']);
        $user_id = intval($_POST['uid']);

        // get the session and end it
        $sessions = new WP_Persistent_Login_Manage_Sessions($user_id);
        $sessions->persistent_login_update_session($token);

        // check if there are more sessions to end
        $invalid_sessions = $this->get_invalid_session_count($user_id);

        // if there are more sessions to end, redirect the user back to the limit reached page
        if( $invalid_sessions > 0 ) {

            $this->redirect_to_login_management($user_id);

        // if not, log the user in
        } else {

            $user = get_user_by('ID', $user_id);

            wp_set_current_user( $user_id, $user->user_login );
            wp_set_auth_cookie( $user_id );
            do_action( 'wp_login', $user->user_login, $user );

            if( isset($_COOKIE['wppl-redirect']) ) {

                if( isset( $_COOKIE['wppl-redirect'] ) ) {
                $redirect_url = $_COOKIE['wppl-redirect'];

                // remove the cookie now, we no longer need it
                unset($_COOKIE['wppl-redirect']); 
                setcookie('wppl-redirect', null, -1, '/'); 
            
            } elseif( get_transient('wppl-redirect-'.$user_id) ) 

                $redirect_url = get_transient('wppl-redirect-'.$user_id);
                delete_transient('wppl-redirect-'.$user_id);

            } else {
            
                $redirect_url = home_url();
            
            }

            /**
             * wppl_login_redirect
             * 
             * (string) $redirect_url = the URL to be redirected to
             * (WP_User object) $user = the user object for conditional filtering
             * 
             * Allows control of where users are redirected to after ending sessions while trying to login.
             */
            $redirect_url = apply_filters('wppl_login_redirect', $redirect_url, $user);

            wp_safe_redirect($redirect_url, 302, 'wp_persistent_login');
            exit;
           

        }

    }


    
    /**
     * init_blocks
     *
     * @return void
     */
    public function init_blocks() {

        if ( ! function_exists( 'register_block_type' ) ) {
            // Gutenberg is not active.
            return;
        }

        // manage sessions block
        wp_register_script(
            'persistent-login-manage-sessions-script',
            WPPL_PLUGIN_URL.'/blocks/manage-sessions/block.js',
            array( 'wp-blocks', 'wp-element' ),
            '1.0.3'
        );
        
        wp_register_style(
            'persistent-login-manage-sessions-editor-style',
            WPPL_PLUGIN_URL.'/blocks/manage-sessions/editor.css',
            array( 'wp-edit-blocks' ),
            '1.0.7' 
        );
        
        wp_register_style(
            'persistent-login-manage-sessions-style',
            WPPL_PLUGIN_URL.'/blocks/manage-sessions/style.css',
            array('wp-block-library'),
            '1.0.1'
        );
        
        register_block_type(
            'persistent-login/manage-sessions', 
            array(
                'style' => 'persistent-login-manage-sessions-style',
                'editor_style' => 'persistent-login-manage-sessions-editor-style',
                'editor_script' => 'persistent-login-manage-sessions-script',
                'render_callback' => array($this, 'render_active_logins')
            )
        );


        // maximum logins control block
        wp_register_script(
            'persistent-login-maximum-logins-control-script',
            WPPL_PLUGIN_URL.'/blocks/maximum-logins-control/block.js',
            array( 'wp-blocks', 'wp-element' ),
            '1.0.3'
        );
        
        wp_register_style(
            'persistent-login-maximum-logins-control-editor-style',
            WPPL_PLUGIN_URL.'/blocks/maximum-logins-control/editor.css',
            array( 'wp-edit-blocks' ),
            '1.0.7' 
        );
        
        wp_register_style(
            'persistent-login-maximum-logins-control-style',
            WPPL_PLUGIN_URL.'/blocks/maximum-logins-control/style.css',
            array('wp-block-table'),
            '1.0.1'
        );
        
        register_block_type(
            'persistent-login/maximum-logins-control', 
            array(
                'style' => 'persistent-login-maximum-logins-controls-style',
                'editor_style' => 'persistent-login-maximum-logins-control-editor-style',
                'editor_script' => 'persistent-login-maximum-logins-control-script',
                'render_callback' => array($this, 'render_maximum_logins_control')
            )
        );

    
    }

    
    /**
     * manage_sessions_shortcode
     *
     * renders the active logins HTML for the shortcode [persistent-login-manage-sessions]
     * and [wppl-manage-sessions]
     * 
     * @return string
     */
    public function manage_sessions_shortcode() {
        return $this->render_active_logins();
    }


    public function render_active_logins() {

        if( is_user_logged_in() ) :
                
            // grab the user
            $user = wp_get_current_user();
            
            // find this session token & create verifier
            $sessionToken = wp_get_session_token();
            $style_tag = '';
                
            if ( function_exists( 'hash' ) ) :
                $verifier = hash('sha256', $sessionToken);
            else :
                $verifier = sha1( $sessionToken);
            endif;	

            if( function_exists('wp_should_load_separate_core_block_assets') ) {
                if( wp_should_load_separate_core_block_assets() === true ) {

                    $handle = 'wp-block-table';
                    $core_styles = file_get_contents( includes_url( "/blocks/table/style.min.css" ) );
                    if( !is_rtl() ) {
                        $theme_styles = file_get_contents( includes_url( "/blocks/table/theme.min.css" ) );
                    } else {
                        $theme_styles = file_get_contents( includes_url( "/blocks/table/theme-rtl.min.css" ) );
                    }
                    
                    $style_tag = "<style>$core_styles $theme_styles</style>";

                }
            }
            
            $output = ''; 
            $output .= $style_tag;
                    
            // remove session if requested
            if( isset($_POST['action']) ) :
                if( $_POST['action'] === 'endSession' ) :
                
                    // setup vars
                    $token = $_POST['token'];
                                    
                    // remove that session
                    $updateSession = new WP_Persistent_Login_Manage_Sessions($user->ID);
                    $updateSession->persistent_login_update_session($token); 
                        
                    // verify that it doesn't exist any more
                    if( !$updateSession->persistent_login_get_session($token) ) :
                        $success_message = __('Session successfully ended.', WPPL_TEXT_DOMAIN);
                        $output .= '<div class="wp-block persistent-login-notification persistent-login-success"><p>'.$success_message.'</p></div>';
                    else :
                        $error_message = __('Failed to end session. Please try again.', WPPL_TEXT_DOMAIN);
                        $output .= '<div class="wp-block persistent-login-notification persistent-login-error"><p>'.$error_message.'</p></div>';
                    endif;
                    
                endif;
            endif;  
            
            
            // start outputting table
            $output .= '<figure class="wp-block-table">';
            $output .= '<table class="wp-block persistent-login-table persistent-login-manage-sessions">';
            
                $output .= '<thead>';
                    $output .= '<tr class="persistent-login-table-header-row">';
                        $output .= '<th>'.__('Session', WPPL_TEXT_DOMAIN).'</th>';
                        $output .= '<th>'.__('Last Visit', WPPL_TEXT_DOMAIN).'</th>';
                        $output .= '<th>'.__('Logout', WPPL_TEXT_DOMAIN).'</th>';
                    $output .= '</tr>';
                $output .= '</thead>';
                    
                $output .= '<tbody>';

                    // get all of the users sessions & sort them by login time
                    $sessions = $this->get_session_data($user);
                        
                    // loop through all of the sessions & output information
                    foreach( $sessions as $session ) :
                                                
                        // output the row
                        $output .= '<tr';
                        if ( $session['current_device'] === true )  :
                            $output .= ' class="persistent-login-this-device"';
                        endif;
                        $output .= '>';
                        
                            // device & IP
                            $output .= '<td title="'. $session['device'] .'">';

                                $output .= $session['device'];
                                if ( $session['current_device'] === true )  :
                                    $output .= '<br/><small class="meta">'.
                                        __('This device', WPPL_TEXT_DOMAIN)
                                    .'</small>';
                                else :
                                    $output .= '<br/><small class="meta">'.
                                        __('IP Address', WPPL_TEXT_DOMAIN).': '. $session['ip']
                                    .'</small>';
                                endif;
                                
                            $output .= '</td>';

                            // Login time
                            $output .= '<td>';
                                $output .= $session['login_time'];
                            $output .= '</td>';

                            // End Session
                            if ( $session['current_device'] === true )  :
                                $output .= '<td>
                                    <a href="'. wp_logout_url() .'" class="persistent-login-end-session-link" title="'.__('Logout of this device', WPPL_TEXT_DOMAIN).'">
                                        <?xml version="1.0" encoding="iso-8859-1"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"> <svg version="1.1" id="Capa_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" width="15px" height="15px" viewBox="0 0 20 20" style="enable-background:new 0 0 19.438 19.438;" xml:space="preserve"> <g> <polygon points="9.72,13.348 15.807,19.438 19.436,15.809 13.35,9.721 19.438,3.632 15.807,0 9.718,6.089 3.63,0 0.001,3.629 6.089,9.718 0,15.807 3.631,19.438 "/> </g></svg>
                                    </a>
                                </td>';
                            else :
                                $output .= '<td>
                                    <form method="POST">
                                        <input type="hidden" name="token" id="token" value="'. $session['session_key'] .'" />
                                        <input type="hidden" name="action" id="action" value="endSession" />
                                        <label title="'.__('End Session', WPPL_TEXT_DOMAIN).'" style="cursor: pointer;" class="persistent-login-end-session-link">
                                            <input type="submit" value="End Session" style="display: none !important;" title="'.__('End Session', WPPL_TEXT_DOMAIN).'" />
                                            <?xml version="1.0" encoding="iso-8859-1"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"> <svg title="'.__('End Session', WPPL_TEXT_DOMAIN).'" version="1.1" id="Capa_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" width="15px" height="15px" viewBox="0 0 20 20" style="enable-background:new 0 0 19.438 19.438;" xml:space="preserve"> <g> <polygon points="9.72,13.348 15.807,19.438 19.436,15.809 13.35,9.721 19.438,3.632 15.807,0 9.718,6.089 3.63,0 0.001,3.629 6.089,9.718 0,15.807 3.631,19.438 "/> </g></svg>
                                        </label>
                                    </form>
                                </td>';
                            endif;
                            // END: End Session

                        $output .= '</tr>';
                    endforeach; 

                $output .= '</tbody>';
            
            $output .= '</table>';
            $output .= '</figure>'; 
                
            
            // return the output
            return $output;
        
        else :
        
            $error_message = __('You need to be logged in to access this information.', WPPL_TEXT_DOMAIN);
            return '<div class="persistent-login-notification persistent-login-error"><p>'.$error_message.'</p></div>';
            
        endif; // login check

    }


    /**
     * maximum_logins_control_shortcode
     *
     * renders the active logins HTML for the shortcode [wppl-maximum-logins-control]
     * 
     * @return string
     */
    public function maximum_logins_control_shortcode() {
        return $this->render_maximum_logins_control();
    }



    public function render_maximum_logins_control() {

        global $wp;
        $url = add_query_arg( $_SERVER['QUERY_STRING'], '', home_url( $wp->request ) );

        // verify that the nonce is correct for this user id
        if( isset($_REQUEST['wppl_nonce']) ) {
            $nonce = $_REQUEST['wppl_nonce'];
            $user_id = (int) $_REQUEST['uid'];
            $nonce_check = wp_verify_nonce( $_REQUEST['wppl_nonce'], 'wppl_end_session_'.$user_id );
        } else {
            $nonce_check = false;
        }

        if( $nonce_check !== false ) {

            $invalid_sessions = $this->get_invalid_session_count($user_id);

            $style_tag = '';
            $sessions_to_end_message = $this->get_sessions_to_end_message($user_id);
            $user = get_user_by('ID', $user_id);
            $sessions = $this->get_session_data($user);

            if( function_exists('wp_should_load_separate_core_block_assets') ) {
                if( wp_should_load_separate_core_block_assets() === true ) {

                    $handle = 'wp-block-table';
                    $core_styles = file_get_contents( includes_url( "/blocks/table/style.min.css" ) );
                    if( !is_rtl() ) {
                        $theme_styles = file_get_contents( includes_url( "/blocks/table/theme.min.css" ) );
                    } else {
                        $theme_styles = file_get_contents( includes_url( "/blocks/table/theme-rtl.min.css" ) );
                    }
                    
                    $style_tag = "<style>$core_styles $theme_styles</style>";

                }
            }
            
            $output = ''; 
            $output .= $style_tag;
            $output .= $sessions_to_end_message;
            $output .= '<figure class="wp-block-table persistent-login-table persistent-login-maximum-logins">';
                $output .= '<table>';

                    $output .= '<thead>';
                        $output .= '<tr class="persistent-login-table-header-row">';
                            $output .= '<th>'. __('Session', WPPL_TEXT_DOMAIN).'</th>';
                            $output .= '<th>'. __('Last Visit', WPPL_TEXT_DOMAIN).'</th>';
                            $output .= '<th>'. __('Logout', WPPL_TEXT_DOMAIN).'</th>';
                        $output .= '</tr>';
                    $output .= '</thead>';

                    $output .= '<tbody>';
                        
                        foreach( $sessions as $session ) :
                            $output .= '<tr>';

                                // device & IP
                                $output .= '<td title="'. $session['device'] .'">';

                                    $output .= $session['device'];
                                    if ( $session['current_device'] === true )  :
                                        $output .= '<br/><small class="meta">'.
                                            __('This device', WPPL_TEXT_DOMAIN)
                                        .'</small>';
                                    else :
                                        $output .= '<br/><small class="meta">'.
                                            __('IP Address', WPPL_TEXT_DOMAIN)
                                        .': '. $session['ip'] .'</small>';
                                    endif;
                                    
                                $output .= '</td>';

                                // Login time
                                $output .= '<td>';
                                    $output .= $session['login_time'];
                                $output .= '</td>';


                                // End Session
                                if ( $session['current_device'] === true )  :
                                    $output .= '<td>
                                        <a href="'. wp_logout_url() .'" class="persistent-login-end-session-link" title="'.__('Logout of this device', WPPL_TEXT_DOMAIN).'">
                                            <?xml version="1.0" encoding="iso-8859-1"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"> <svg version="1.1" id="Capa_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" width="15px" height="15px" viewBox="0 0 20 20" style="enable-background:new 0 0 19.438 19.438;" xml:space="preserve"> <g> <polygon points="9.72,13.348 15.807,19.438 19.436,15.809 13.35,9.721 19.438,3.632 15.807,0 9.718,6.089 3.63,0 0.001,3.629 6.089,9.718 0,15.807 3.631,19.438 "/> </g></svg>
                                        </a>
                                    </td>';
                                else :
                                    $output .= '<td>
                                        <form method="POST" action="'. esc_url( admin_url('admin-post.php') ) .'">
                                            <input type="hidden" name="token" id="token" value="'. $session['session_key'] .'" />
                                            <input type="hidden" name="uid" id="uid" value="'. $user_id .'" />
                                            <input type="hidden" name="action" value="wppl_end_session" />
                                            '. wp_nonce_field('wppl_end_session', '_wpnonce', true, false ) .'

                                            <label title="'.__('End Session', WPPL_TEXT_DOMAIN).'" style="cursor: pointer;" class="persistent-login-end-session-link">
                                                <input type="submit" value="End Session" style="display: none !important;" title="'.__('End Session', WPPL_TEXT_DOMAIN).'" />
                                                <?xml version="1.0" encoding="iso-8859-1"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"> <svg title="'.__('End Session', WPPL_TEXT_DOMAIN).'" version="1.1" id="Capa_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" width="15px" height="15px" viewBox="0 0 20 20" style="enable-background:new 0 0 19.438 19.438;" xml:space="preserve"> <g> <polygon points="9.72,13.348 15.807,19.438 19.436,15.809 13.35,9.721 19.438,3.632 15.807,0 9.718,6.089 3.63,0 0.001,3.629 6.089,9.718 0,15.807 3.631,19.438 "/> </g></svg>
                                            </label>
                                        </form>
                                    </td>';
                                endif;
                                // END: End Session

                            $output .= '</tr>';
                        endforeach; 

                    $output .= '</tbody>';

                $output .= '</table>';
            $output .= '</figure>';
        
        } else {
            
            if( is_user_logged_in() ) {
                $error_message = __('<p>You haven\'t reached the active login limit.</p> <p>Please try again or continue browsing the website.</p>', WPPL_TEXT_DOMAIN); 
            } else {
                $error_message = __('<p>Sorry, you don\'t have permission to view this data.</p> <p>Please try again or continue browsing the website.</p>', WPPL_TEXT_DOMAIN); 
            }
            
            $output = $error_message;
            
        }

        return $output;

    }


}


?>